/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.padre;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import org.apache.commons.codec.binary.Base64;
import org.bbaw.pdr.padre.request.PadreID;
import org.bbaw.pdr.padre.request.PadreObject;
import org.bbaw.pdr.utils.Identifier;
import org.bbaw.pdr.utils.PDRObject;
import org.bbaw.pdr.utils.PDRType;

/**
 * Returns modified objects since provided timestamp.
 * @author Fabian Körner
 */
public final class PadreGetModified extends AbstractPadreCommand<List<PDRObject>>
{

	private int  _store;
	private String _timestamp;

	/**
	 * Reads objects from provided PaDRe store which were modified
	 * since given timestamp.
	 * @param store Requested PaDRe store.
	 * @param timestamp Timestamp.
	 * @throws PadreCommandException Could not get modified objects.
	 */
	public PadreGetModified(final int store, final String timestamp) throws PadreCommandException
	{
		super();
		_store = store;
		_timestamp = timestamp;
	}

	/**
	 * @see org.bbaw.pdr.padre.AbstractPadreCommand#execute()
	 * @return Modified PDR objects.
	 * @throws PadreCommandException Could not successfully execute PaDRe command.
	 */
	@Override
	public List<PDRObject> execute() throws PadreCommandException
	{
		List<PDRObject> result = new Vector<PDRObject>();
		try
		{
			String storeID = Identifier.intToHexString(_store, 3);
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
			Date since = sdf.parse(_timestamp);

			List<PadreID> padreIDs = new Vector<PadreID>();
			for (PDRType type : PDRType.values())
			{
				padreIDs.addAll(super.getPadreClient().modified(storeID, type.toString(), since));
			}

			if (!padreIDs.isEmpty())
			{
				List<PadreObject> padreObjects = super.getPadreClient().read(padreIDs);
				for (PadreObject padreObject : padreObjects)
				{
					String payload = padreObject.getPayload();
					byte[] content = Base64.decodeBase64(payload);
					result.add(new PDRObject(new String(content, "UTF-8")));
				}
			}
		}
		catch (Exception cause)
		{
			throw new PadreCommandException(cause);
		}
		return result;
	}
}
