/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.padre.test;

import static org.junit.Assert.assertEquals;

import java.util.List;
import java.util.Vector;

import org.bbaw.pdr.padre.PadreConfiguration;
import org.bbaw.pdr.padre.PadreGetModified;
import org.bbaw.pdr.padre.client.PadreClient;
import org.bbaw.pdr.padre.request.PadreID;
import org.bbaw.pdr.padre.request.PadreObject;
import org.bbaw.pdr.padre.request.PadreObjectException;
import org.bbaw.pdr.utils.PDRObject;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
*
* @author Fabian Körner
*/
public class TestPadreGetModified
{

	private static final TestConfiguration CONFIGURATION = new TestConfigurationFromFile();

	private PadreClient _padreClient;

	/**
	 * @throws Exception Exception.
	 */
	@Before
	public final void setUp() throws Exception
	{
		PadreConfiguration.getInstance().setCredentials(CONFIGURATION.getUri(), CONFIGURATION.getUsername(), CONFIGURATION.getPassword());
		_padreClient = PadreConfiguration.getInstance().getPadreClient();

		_padreClient.createStore("31e");
	}

	/**
	 * @throws Exception Exception.
	 */
	@After
	public final void tearDown() throws Exception
	{
		_padreClient.deleteStore("31e");
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testGetAll() throws Exception
	{
		String timestamp = _padreClient.time();

		List<PadreObject> testObjects = new Vector<PadreObject>();
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000001")));
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000002")));
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000003")));
		_padreClient.write(testObjects);

		List<PDRObject> result = new PadreGetModified(798, timestamp).execute();

		assertEquals(3, result.size());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testGetAllWithPause() throws Exception
	{
		String timestamp = _padreClient.time();
		TestPadreGetModified.pause(1000L);

		List<PadreObject> testObjects = new Vector<PadreObject>();
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000001")));
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000002")));
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000003")));
		_padreClient.write(testObjects);

		List<PDRObject> result = new PadreGetModified(798, timestamp).execute();

		assertEquals(3, result.size());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testGetPart() throws Exception
	{
		List<PadreObject> testObjects = new Vector<PadreObject>();
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000001")));
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000002")));
		testObjects.add(TestPadreGetModified.createTestObject(new PadreID("pdrAo.001.31e.000000003")));
		_padreClient.write(testObjects);

		TestPadreGetModified.pause(1000L);

		String timestamp = _padreClient.time();

		testObjects.clear();
		testObjects.add(TestPadreGetModified.createModifiedTestObject(new PadreID("pdrAo.001.31e.000000002")));
		testObjects.add(TestPadreGetModified.createModifiedTestObject(new PadreID("pdrAo.001.31e.000000004")));
		_padreClient.write(testObjects);

		List<PDRObject> result = new PadreGetModified(798, timestamp).execute();

		assertEquals(2, result.size());
	}

	private static PadreObject createTestObject(final PadreID padreID) throws PadreObjectException
	{
		return new PadreObject(padreID, "PHBlcnNvbiB4bWxucz0iaHR0cDovL3Bkci5iYmF3LmRlL25hbWVzcGFjZXMvcG9kbC8iIHhtbG5zOnhzaT0iaHR0cDovL3d3dy53My5vcmcvMjAwMS9YT"
				+ "UxTY2hlbWEtaW5zdGFuY2UiIHhzaTpzY2hlbWFMb2NhdGlvbj0iaHR0cDovL3Bkci5iYmF3LmRlL25hbWVzcGFjZXMvcG9kbC8gaHR0cDovL3Bkci5iYmF3LmRlL3NjaGVtYS9wb2RsL"
				+ "nhzZCIgaWQ9InBkclBvLjAwMS43OTguMDAwMDAwMDAxIj48cmVjb3JkPjxyZXZpc2lvbiByZWY9IjAiIHRpbWVzdGFtcD0iMjAxNC0wMS0xM1QxNjoxNDoxNiIgYXV0aG9yaXR5PSJwZ"
				+ "HJVby4wMDEuNzg5LjAwMDAwMDAxMCIvPjwvcmVjb3JkPjwvcGVyc29uPg==",
				"c18423cc7054d59e7d2a2df1a555dc0e7408acfd57a874d5f705b397b4e235b4");
	}

	private static PadreObject createModifiedTestObject(final PadreID padreID) throws PadreObjectException
	{
		return new PadreObject(padreID, "PHBlcnNvbiB4bWxucz0iaHR0cDovL3Bkci5iYmF3LmRlL25hbWVzcGFjZXMvcG9kbC8iIHhtbG5zOnhzaT0iaHR0cDovL3d3dy53My5vcmcvMjAwMS9YT"
				+ "UxTY2hlbWEtaW5zdGFuY2UiIHhzaTpzY2hlbWFMb2NhdGlvbj0iaHR0cDovL3Bkci5iYmF3LmRlL25hbWVzcGFjZXMvcG9kbC8gaHR0cDovL3Bkci5iYmF3LmRlL3NjaGVtYS9wb2RsL"
				+ "nhzZCIgaWQ9InBkclBvLjAwMS43OTguMDAwMDAwMDAxIj48cmVjb3JkPjxyZXZpc2lvbiByZWY9IjAiIHRpbWVzdGFtcD0iMjAxNC0wMS0xM1QxNjoxNDoxNiIgYXV0aG9yaXR5PSJwZ"
				+ "HJVby4wMDEuNzg5LjAwMDAwMDAxMSIvPjwvcmVjb3JkPjwvcGVyc29uPg==",
				"55a906bad88651eb3da95cc2d50825cf6d46650c51596f55e01891275f44cbe7");
	}

	private static void pause(final long millis)
	{
		try
		{
			Thread.sleep(millis);
		}
		catch (InterruptedException e)
		{
			/* ignore */
		}
	}
}
