/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.solr;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Queue;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.solr.common.SolrInputDocument;
import org.bbaw.pdr.utils.Identifier;

/**
 * Solr input document with common index fields.
 * @author Fabian Körner
 */
public class SolrInputCommon extends SolrInputDocument
{

	private static final long serialVersionUID = 923977151972572800L;

	private OMElement _root;
	private String _defaultNamespaceURI;
	private String _defaultNamespacePrefix;

	/* Elemente */
	private QName _qNameRecordElement;
	private QName _qNameRevisionElement;
	private QName _qNameRecordInfoElement;
	private QName _qNameRecordCreationDateElement;
	private QName _qNameRecordChangeDateElement;
	private QName _qNameRecordContentSourceElement;

	/* Attribute */
	private QName _qNameIdAttributeCommon;
	private QName _qNameIdAttributeReference;
	private QName _qNameTimestampAttribute;
	private QName _qNameAuthorityAttribute;

	protected SolrInputCommon(final OMElement root)
	{
		super();
		_root = root;
		_defaultNamespaceURI = _root.getDefaultNamespace().getNamespaceURI();
		_defaultNamespacePrefix = _root.getDefaultNamespace().getPrefix();
		this.initQNames();
		this.addCommonIndexFields();
	}

	private void initQNames()
	{
		_qNameRecordElement = createQNameOfElement("record");
		_qNameRevisionElement = createQNameOfElement("revision");
		_qNameRecordInfoElement = createQNameOfElement("recordInfo");
		_qNameRecordCreationDateElement = createQNameOfElement("recordCreationDate");
		_qNameRecordChangeDateElement = createQNameOfElement("recordChangeDate");
		_qNameRecordContentSourceElement = createQNameOfElement("recordContentSource");

		_qNameIdAttributeCommon = createQNameOfAttribute("id");
		_qNameIdAttributeReference = createQNameOfAttribute("ID");
		_qNameTimestampAttribute = createQNameOfAttribute("timestamp");
		_qNameAuthorityAttribute = createQNameOfAttribute("authority");
	}

	protected final QName createQNameOfElement(final String localName)
	{
		return new QName(_defaultNamespaceURI, localName, _defaultNamespacePrefix);
	}

	protected final QName createQNameOfAttribute(final String localName)
	{
		return new QName(localName);
	}

	protected final OMElement getRoot()
	{
		return _root;
	}

	private void addCommonIndexFields()
	{
		String idAttributeValue = getIDAttributeValueFromRootElement();

		Identifier id = Identifier.create(idAttributeValue);

		Map<String, String> revisions = getRevisions();

		addIDFields(id);
		addRevisionFields(revisions);
	}

	private String getIDAttributeValueFromRootElement()
	{
		String rootName = _root.getLocalName();
		if (SolrInputDocumentFactory.LOCAL_NAME_ROOT_REFERENCE.equals(rootName))
		{
			return _root.getAttributeValue(_qNameIdAttributeReference);
		}
		else
		{
			return _root.getAttributeValue(_qNameIdAttributeCommon);
		}
	}

	private Map<String, String> getRevisions()
	{
		String rootName = _root.getLocalName();
		if (SolrInputDocumentFactory.LOCAL_NAME_ROOT_REFERENCE.equals(rootName))
		{
			return getRevisionsFromReference();
		}
		else
		{
			Map<String, String> result = new Hashtable<String, String>();
			OMElement record = _root.getFirstChildWithName(_qNameRecordElement);

			OMElement revision;
			String timestamp;
			String authority;
			for (Iterator<?> i = record.getChildrenWithName(_qNameRevisionElement); i.hasNext();)
			{
				revision = (OMElement) i.next();
				timestamp = revision.getAttributeValue(_qNameTimestampAttribute);
				authority = revision.getAttributeValue(_qNameAuthorityAttribute);
				result.put(timestamp, authority);
			}
			return result;
		}
	}

	private Map<String, String> getRevisionsFromReference()
	{
		Map<String, String> result = new Hashtable<String, String>();
		Iterator<?> recordInfos = _root.getChildrenWithName(_qNameRecordInfoElement);

		OMElement recordInfo;
		OMElement recordCreationDate;
		OMElement recordChangeDate;
		OMElement recordContentSource;
		String timestamp;
		String authority;
		while (recordInfos.hasNext())
		{
			recordInfo = (OMElement) recordInfos.next();
			recordCreationDate = recordInfo.getFirstChildWithName(_qNameRecordCreationDateElement);
			recordChangeDate = recordInfo.getFirstChildWithName(_qNameRecordChangeDateElement);
			if (null == recordCreationDate)
			{
				timestamp = recordChangeDate.getText();
			}
			else
			{
				timestamp = recordCreationDate.getText();
			}
			recordContentSource = recordInfo.getFirstChildWithName(_qNameRecordContentSourceElement);
			authority = recordContentSource.getText();
			result.put(timestamp, authority);
		}
		return result;
	}

	private void addIDFields(final Identifier id)
	{
		addField("id", id.toString());
		addField("id.type", id.getType().toString());
		addField("id.instance", Integer.toString(id.getInstanceNumber()));
		addField("id.project", Integer.toString(id.getProjectNumber()));
		addField("id.object", Integer.toString(id.getObjectNumber()));
	}

	private void addRevisionFields(final Map<String, String> revisions)
	{
		String timestamp;
		String authority;
		String cRevision;
		for (Entry<String, String> entry : revisions.entrySet())
		{
			timestamp = entry.getKey() + "Z";
			authority = entry.getValue();
			cRevision = getCombinedIndexValue(timestamp, authority);
			addField("revision.timestamp", timestamp);
			addField("revision.authority", authority);
			addField("crevision", cRevision);
		}
	}

	protected static String getCombinedIndexValue(final String... values)
	{
		StringBuilder sb = new StringBuilder();
		for (String value : values)
		{
			sb.append(value);
			sb.append('|');
		}
		sb.deleteCharAt(sb.length() - 1);
		return sb.toString();
	}

	protected final void addFields(final Queue<Field> fields)
	{
		Field currentField;
		String currentName;
		String currentValue;
		while (null != (currentField = fields.poll()))
		{
			currentName = currentField.getName();
			currentValue = currentField.getValue();
			if (!currentValue.isEmpty())
			{
				addField(currentName, currentValue);
			}
		}
	}

	/**
	 * Encapsulates name and value of an index field.
	 * @author Fabian Körner
	 */
	protected class Field
	{
		private String _name;
		private String _value;

		protected Field(final String name, final String value)
		{
			_name = name;
			_value = value;
		}

		protected final String getName()
		{
			return _name;
		}

		protected final String getValue()
		{
			return _value;
		}
	}
}
