/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.solr.test;

import static org.junit.Assert.assertEquals;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.solr.client.solrj.SolrQuery;
import org.apache.solr.client.solrj.SolrQuery.ORDER;
import org.apache.solr.client.solrj.SolrServer;
import org.apache.solr.client.solrj.impl.HttpSolrServer;
import org.apache.solr.client.solrj.response.QueryResponse;
import org.apache.solr.client.solrj.response.SolrPingResponse;
import org.apache.solr.client.solrj.response.UpdateResponse;
import org.apache.solr.common.SolrDocumentList;
import org.apache.solr.common.SolrInputDocument;
import org.bbaw.pdr.solr.SolrConfiguration;
import org.bbaw.pdr.solr.SolrHttpQuery;
import org.bbaw.pdr.solr.SolrPlainQuery;
import org.bbaw.pdr.utils.Identifier;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * @author Fabian Körner
 */
public class TestSolr
{

	private static final TestConfiguration CONFIGURATION = new TestConfigurationFromFile();
	private static SolrServer _server;

	/**
	 * @throws Exception Exception.
	 */
	@BeforeClass
	public static void setUp() throws Exception
	{
		SolrConfiguration.getInstance().setBaseURI(CONFIGURATION.getUri());
		_server = SolrConfiguration.getInstance().getSolrServer();
		_server.add(TestSolr.createSimplePerson(new Identifier("pdrPo.001.789.000000001")));
		_server.add(TestSolr.createSimplePerson(new Identifier("pdrPo.001.789.000000002")));
		_server.add(TestSolr.createSimplePerson(new Identifier("pdrPo.001.789.000000003")));
		_server.add(TestSolr.createSimplePerson(new Identifier("pdrPo.001.789.000000004")));
		_server.add(TestSolr.createSimplePerson(new Identifier("pdrPo.001.789.000000005")));
		_server.commit();
	}

	/**
	 * @throws Exception Exception.
	 */
	@AfterClass
	public static final void tearDown() throws Exception
	{
		_server.deleteByQuery("id.project:789");
		((HttpSolrServer) _server).shutdown();
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testPing() throws Exception
	{
		SolrPingResponse response = _server.ping();
		assertEquals(0, response.getStatus());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testWrite() throws Exception
	{
		/* single */
		UpdateResponse addSingleResponse = _server.add(TestSolr.createSimpleAspect());

		/* collection */
		Collection<SolrInputDocument> inputDocuments = new Vector<SolrInputDocument>();
		inputDocuments.add(TestSolr.createPersonWithPND());
		inputDocuments.add(TestSolr.createPersonWithTwoExternalIDs());
		UpdateResponse addCollectionResponse = _server.add(inputDocuments);

		assertEquals(0, addSingleResponse.getStatus());
		assertEquals(0, addCollectionResponse.getStatus());

		UpdateResponse commitResponse = _server.commit();
		assertEquals(0, commitResponse.getStatus());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testIDQuery() throws Exception
	{
		String expectedList = "{numFound=1,start=0,docs=[SolrDocument[{id=pdrPo.001.789.000000002}]]}";

		String query = "id:pdrPo.001.789.000000002";
		SolrQuery solrQuery = new SolrQuery(query);

		QueryResponse response = new SolrPlainQuery(solrQuery).execute();
		SolrDocumentList actualList = response.getResults();

		assertEquals(1, response.getResults().size());
		assertEquals(expectedList, actualList.toString());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testIDPartsQuery() throws Exception
	{
		String expectedList = "{numFound=1,start=0,docs=[SolrDocument[{id=pdrPo.001.789.000000003}]]}";

		String query = "id.type:pdrPo AND id.instance:1 AND id.project:789 AND id.object:3";
		SolrQuery solrQuery = new SolrQuery(query);

		QueryResponse response = new SolrPlainQuery(solrQuery).execute();
		SolrDocumentList actualList = response.getResults();

		assertEquals(1, response.getResults().size());
		assertEquals(expectedList, actualList.toString());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testIDParts2Query() throws Exception
	{
		String expectedList = "{numFound=2,start=0,docs=[SolrDocument[{id=pdrPo.001.789.000000001}], SolrDocument[{id=pdrPo.001.789.000000003}]]}";

		String query = "id.type:pdrPo AND id.instance:1 AND id.project:789 AND (id.object:3 OR id.object:1)";
		SolrQuery solrQuery = new SolrQuery(query);
		solrQuery.setSortField("id.object", ORDER.asc);

		QueryResponse response = new SolrPlainQuery(solrQuery).execute();
		SolrDocumentList sdl = response.getResults();

		assertEquals(2, response.getResults().size());
		assertEquals(expectedList, sdl.toString());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testDelete() throws Exception
	{
		UpdateResponse deleteResponse = _server.deleteByQuery("id.project:789");
		assertEquals(0, deleteResponse.getStatus());

		UpdateResponse commitResponse = _server.commit();
		assertEquals(0, commitResponse.getStatus());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (timeout = 2000)
	public final void testHttpQuery() throws Exception
	{
		Map<String, String> params = new HashMap<String, String>();
		params.put("q", "*:*");
		params.put("version", "2.2");
		params.put("start", "0");
		params.put("rows", "10");
		params.put("indent", "on");
		String result = new SolrHttpQuery(params).execute();

		assertEquals(2, result.indexOf("xml"));
		assertEquals(62, result.indexOf("responseHeader"));
		assertEquals(92, result.indexOf("status"));
		assertEquals(121, result.indexOf("QTime"));
	}

	private static SolrInputDocument createSimpleAspect()
	{
		SolrInputDocument result = new SolrInputDocument();
		result.addField("id", "pdrAo.001.789.000002979");
		result.addField("id.type", "pdrAo");
		result.addField("id.instance", 1);
		result.addField("id.project", 789);
		result.addField("id.object", 2979);
		result.addField("notification", "narf");
		return result;
	}

	private static SolrInputDocument createSimplePerson(final Identifier id)
	{
		SolrInputDocument result = new SolrInputDocument();
		result.addField("id", id.toString());
		result.addField("id.type", id.getType().toString());
		result.addField("id.instance", Integer.toString(id.getInstanceNumber(), 10));
		result.addField("id.project", Integer.toString(id.getProjectNumber(), 10));
		result.addField("id.object", Integer.toString(id.getObjectNumber(), 10));
		return result;
	}

	private static SolrInputDocument createPersonWithPND()
	{
		SolrInputDocument result = new SolrInputDocument();
		result.addField("id", "pdrPo.001.789.000002972");
		result.addField("id.type", "pdrPo");
		result.addField("id.instance", 1);
		result.addField("id.project", 789);
		result.addField("id.object", 2972);
		result.addField("person.identifier", "123456789X");
		result.addField("person.identifier.provider", "DNB");
		result.addField("person.cidentifier", "123456789X|DNB");
		return result;
	}

	private static SolrInputDocument createPersonWithTwoExternalIDs()
	{
		SolrInputDocument result = new SolrInputDocument();
		result.addField("id", "pdrPo.001.789.000002973");
		result.addField("id.type", "pdrPo");
		result.addField("id.instance", 1);
		result.addField("id.project", 789);
		result.addField("id.object", 2973);
		result.addField("person.identifier", "123456789X");
		result.addField("person.identifier.provider", "DNB");
		result.addField("person.cidentifier", "123456789X|DNB");
		result.addField("person.identifier", "100340");
		result.addField("person.identifier.provider", "nobody");
		result.addField("person.cidentifier", "100340|nobody");
		return result;
	}

}
