/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Represents a PDR object and its XML format with whilst taking into account
 * that IDs have to be modified under certain cirumstances.
 * @author Fabian Körner
 */
public class PDRObject
{

	private static final SimpleDateFormat DATE_FORMAT_TIMESTAMP = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

	private static final Pattern PATTERN_OBJECT_ID = Pattern.compile("\\s(id|ID)=[\"](.+?)[\"]");
	private static final Pattern PATTERN_REVISION_TIMESTAMP = Pattern.compile("<revision.+?timestamp=[\"](.+?)[\"]");
	private static final Pattern PATTERN_RECORDINFO_TIMESTAMP = Pattern.compile("<record(Creation|Change)Date\\s.+?>(.+?)</record(Creation|Change)Date>");

	private Identifier _originalIdentifier = null;
	private Identifier _modifiedIdentifier = null;
	private String _xmlContent = null;

	/**
	 * Constructor with XML formatted PDR object.
	 * @param xmlContent XML formatted PDR object.
	 * @throws ValidationException Could not instantiate PDR object.
	 */
	public PDRObject(final String xmlContent) throws ValidationException
	{
		_xmlContent = xmlContent;
		this.findOriginalIdentifier();
		this.cleanXMLContent();
		Validator.validate(this);
	}

	/**
	 * Replaces all occurances of the current PDR ID by the provided ID.
	 * @param newIdentifier New ID.
	 */
	public final synchronized void modifyIdentifier(final Identifier newIdentifier)
	{
		if (_modifiedIdentifier == null)
		{
			_xmlContent.replaceAll(_originalIdentifier.toString(), newIdentifier.toString());
		}
		else
		{
			_xmlContent.replaceAll(_modifiedIdentifier.toString(), newIdentifier.toString());
		}
		_modifiedIdentifier = newIdentifier;
	}

	/**
	 * Returns the modified PDR ID of the object. If there was no modification, the
	 * original ID will be returned.
	 * @return Modified (or original) ID of the object.
	 */
	public final synchronized Identifier getModifiedIdentifier()
	{
		return (_modifiedIdentifier == null) ? _originalIdentifier : _modifiedIdentifier;
	}

	/**
	 * Returns the original ID of the object.
	 * @return Original ID.
	 */
	public final synchronized Identifier getOriginalIdentifier()
	{
		return _originalIdentifier;
	}

	/**
	 * Returns the instance number of the object.
	 * @return Object's instance ID.
	 */
	public final int getInstance()
	{
		return this.getModifiedIdentifier().getInstanceNumber();
	}

	/**
	 * Returns the type of the object.
	 * @return Object's type.
	 */
	public final PDRType getType()
	{
		return this.getModifiedIdentifier().getType();
	}

	/**
	 * Return the project number of the object.
	 * @return Object's project ID.
	 */
	public final int getProject()
	{
		return this.getModifiedIdentifier().getProjectNumber();
	}

	/**
	 * Liefert die XML-Repräsentation des Objektes.
	 * @return XML-Repräsentation des Objektes.
	 */
	public final synchronized String getXMLContent()
	{
		return _xmlContent;
	}

	/**
	 * Sets object's XML content. This method has to be used carefully, because
	 * it will neither validate the document nor do any cleansing.
	 * @param xmlContent XML-Repräsentation des Objektes.
	 */
	public final synchronized void setXMLContent(final String xmlContent)
	{
		_xmlContent = xmlContent;
	}

	/**
	 * Returns dates from all timestamps of the object's revisions.
	 * @return Timestamps of revisions.
	 * @throws ParseException Invalid timestamp.
	 */
	public final synchronized List<Date> getRevisionTimestamps() throws ParseException
	{
		final List<Date> result = new Vector<Date>();
		final Identifier identifier = (_modifiedIdentifier == null) ? _originalIdentifier : _modifiedIdentifier;
		Matcher m;
		switch (identifier.getType())
		{
		case REFERENCE:
			m = PATTERN_RECORDINFO_TIMESTAMP.matcher(_xmlContent);
			while (m.find())
			{
				result.add(DATE_FORMAT_TIMESTAMP.parse(m.group(2)));
			}
			break;
		default:
			m = PATTERN_REVISION_TIMESTAMP.matcher(_xmlContent);
			while (m.find())
			{
				result.add(DATE_FORMAT_TIMESTAMP.parse(m.group(1)));
			}
		}
		return result;
	}

	private void findOriginalIdentifier() throws ValidationException
	{
		Matcher m = PATTERN_OBJECT_ID.matcher(_xmlContent);
		if (m.find())
		{
			_originalIdentifier = new Identifier(m.group(2));
		}
		else
		{
			throw new ValidationException("could not find object's identifier in xml content: " + _xmlContent);
		}
	}

	private void cleanXMLContent()
	{
		this.removeNeedlessWhitespaces();
		this.removeXMLDeclaration();
		this.removeNeedlessNamespace();
	}

	private void removeNeedlessWhitespaces()
	{
		_xmlContent = _xmlContent.trim();
		_xmlContent = _xmlContent.replaceAll("\n", "");
		_xmlContent = _xmlContent.replaceAll("\r", "");
	}

	private void removeXMLDeclaration()
	{
		_xmlContent = _xmlContent.replaceAll("<[?]xml.+?[?]>", "");
	}

	private void removeNeedlessNamespace()
	{
		_xmlContent = _xmlContent.replaceAll("[:](mods|[apu]odl)", "");
		_xmlContent = _xmlContent.replaceAll("(mods|[apu]odl)[:]", "");
	}
}
