/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.utils.test;

import static org.junit.Assert.assertEquals;

import java.util.List;
import java.util.Vector;

import org.bbaw.pdr.utils.IDRange;
import org.bbaw.pdr.utils.PDRType;
import org.bbaw.pdr.utils.ValidationException;
import org.junit.Test;

/**
 * @author Fabian Körner
 */
public class TestIDRange
{

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testIDRange() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 123, 456, 23, 36);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (expected = ValidationException.class)
	public final void testInvalidIDRangeInstanceTooLow() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 0, 456, 1, 2);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (expected = ValidationException.class)
	public final void testInvalidIDRangeProjectTooLow() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 123, 0, 3, 6);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (expected = ValidationException.class)
	public final void testInvalidIDRangeBoundsTooLow() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 123, 456, 0, 0);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (expected = ValidationException.class)
	public final void testInvalidIDRangeInstanceTooHigh() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 1000, 456, 200000000, 200000001);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (expected = ValidationException.class)
	public final void testInvalidIDRangeProjectTooHigh() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 123, 4560, 200000000, 200000001);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (expected = ValidationException.class)
	public final void testInvalidIDRangeBoundsTooHigh() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 123, 456, 200000000, 200000001);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test (expected = ValidationException.class)
	public final void testInvalidIDRangeUpperBoundLowerThanLowerBound() throws Exception
	{
		new IDRange(PDRType.REFERENCE, 123, 456, 23, 16);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testGetComponents() throws Exception
	{
		IDRange range = new IDRange(PDRType.REFERENCE, 123, 456, 23, 36);
		assertEquals(PDRType.REFERENCE, range.getType());
		assertEquals(123, range.getInstance());
		assertEquals(456, range.getProject());
		assertEquals(23, range.getLowerBound());
		assertEquals(36, range.getUpperBound());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testCount() throws Exception
	{
		IDRange range = new IDRange(PDRType.REFERENCE, 123, 456, 23, 36);
		assertEquals(14, range.count());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testToString() throws Exception
	{
		IDRange range = new IDRange(PDRType.REFERENCE, 123, 456, 23, 36);
		assertEquals("IDRange[pdrRo,123,456,23-36]", range.toString());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testSafeLongToInt0() throws Exception
	{
		assertEquals(0, IDRange.safeLongToInt(0L));
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testSafeLongToIntPlus1() throws Exception
	{
		assertEquals(1, IDRange.safeLongToInt(1L));
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testSafeLongToIntMinus1() throws Exception
	{
		assertEquals(-1, IDRange.safeLongToInt(-1L));
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testSafeLongToInt12345() throws Exception
	{
		assertEquals(12345, IDRange.safeLongToInt(12345L));
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testSafeLongToIntMin() throws Exception
	{
		assertEquals(-2147483648, IDRange.safeLongToInt(new Long(Integer.toString(Integer.MIN_VALUE)).longValue()));
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testSafeLongToIntMax() throws Exception
	{
		assertEquals(2147483647, IDRange.safeLongToInt(new Long(Integer.toString(Integer.MAX_VALUE)).longValue()));
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testSafeLongToIntTooLow() throws Exception
	{
		IDRange.safeLongToInt(new Long("-2147483649").longValue());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testSafeLongToIntTooHigh() throws Exception
	{
		IDRange.safeLongToInt(new Long("2147483648").longValue());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testSafeLongToIntMuchTooLow() throws Exception
	{
		IDRange.safeLongToInt(Long.MIN_VALUE);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test(expected = IllegalArgumentException.class)
	public final void testSafeLongToIntMuchTooHigh() throws Exception
	{
		IDRange.safeLongToInt(Long.MAX_VALUE);
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges1() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 1, 1));
		result = IDRange.applyLimitsToRanges(ranges, 3, 5);
		assertEquals(0, result.size());

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 7, 7));
		result = IDRange.applyLimitsToRanges(ranges, 3, 5);
		assertEquals(0, result.size());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges2() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 1, 2));
		result = IDRange.applyLimitsToRanges(ranges, 3, 5);
		assertEquals(0, result.size());

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 7, 10));
		result = IDRange.applyLimitsToRanges(ranges, 3, 5);
		assertEquals(0, result.size());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges3() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 4, 4));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(4, result.get(0).getLowerBound());
		assertEquals(4, result.get(0).getUpperBound());

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 8, 8));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(8, result.get(0).getLowerBound());
		assertEquals(8, result.get(0).getUpperBound());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges4() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 2, 4));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(4, result.get(0).getLowerBound());
		assertEquals(4, result.get(0).getUpperBound());

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 8, 10));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(8, result.get(0).getLowerBound());
		assertEquals(8, result.get(0).getUpperBound());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges5() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 2, 5));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(4, result.get(0).getLowerBound());
		assertEquals(5, result.get(0).getUpperBound());

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 6, 10));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(6, result.get(0).getLowerBound());
		assertEquals(8, result.get(0).getUpperBound());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges6() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 6, 6));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(6, result.get(0).getLowerBound());
		assertEquals(6, result.get(0).getUpperBound());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges7() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 5, 7));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(5, result.get(0).getLowerBound());
		assertEquals(7, result.get(0).getUpperBound());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges8() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 4, 8));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(4, result.get(0).getLowerBound());
		assertEquals(8, result.get(0).getUpperBound());
	}

	/**
	 * @throws Exception Exception.
	 */
	@Test
	public final void testApplyLimitsToRanges9() throws Exception
	{
		List<IDRange> ranges;
		List<IDRange> result;

		ranges = new Vector<IDRange>();
		ranges.add(new IDRange(PDRType.ASPECT, 1, 1, 2, 10));
		result = IDRange.applyLimitsToRanges(ranges, 4, 8);
		assertEquals(1, result.size());
		assertEquals(4, result.get(0).getLowerBound());
		assertEquals(8, result.get(0).getUpperBound());
	}
}
