/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.padre;

import java.util.List;
import java.util.Vector;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.bbaw.pdr.padre.request.PadreID;
import org.bbaw.pdr.padre.request.PadreObject;
import org.bbaw.pdr.utils.Identifier;
import org.bbaw.pdr.utils.PDRObject;

/**
 * Writes 1..n PDR objects to PaDRe.
 * @author Fabian Körner
 */
public final class PadreWriteXml extends AbstractPadreCommand<Object>
{

	private List<PDRObject> _pdrObjects;

	/**
	 * Writes the provided single PDR object to PaDRe.
	 * @param pdrObject Object to be written.
	 * @throws PadreCommandException Could not write to PaDRe.
	 */
	public PadreWriteXml(final PDRObject pdrObject) throws PadreCommandException
	{
		super();
		_pdrObjects = new Vector<PDRObject>();
		_pdrObjects.add(pdrObject);
	}

	/**
	 * Writes the provided list of PDR objects to PaDRe.
	 * @param pdrObjects Objects to be written.
	 * @throws PadreCommandException Could not write to PaDRe.
	 */
	public PadreWriteXml(final List<PDRObject> pdrObjects) throws PadreCommandException
	{
		super();
		_pdrObjects = pdrObjects;
	}

	/**
	 * @see org.bbaw.pdr.padre.AbstractPadreCommand#execute()
	 * @return Null.
	 * @throws PadreCommandException Could not successfully execute PaDRe command.
	 */
	@Override
	public Object execute() throws PadreCommandException
	{
		try
		{
			List<PadreObject> padreObjects = new Vector<PadreObject>();
			for (PDRObject pdrObject : _pdrObjects)
			{
				String content = pdrObject.getXMLContent();
				String payload = Base64.encodeBase64String(content.getBytes("UTF-8"));
				String sha256hex = DigestUtils.sha256Hex(payload);

				Identifier identifier = pdrObject.getModifiedIdentifier();
				PadreID padreID = identifier.toPadreID();

				PadreObject padreObject = new PadreObject(padreID, payload, sha256hex);
				padreObjects.add(padreObject);
			}
			super.getPadreClient().write(padreObjects);
		}
		catch (Exception cause)
		{
			throw new PadreCommandException(cause);
		}
		return null;
	}
}
