/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PaDRe.
 *
 *	PaDRe is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PaDRe is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PaDRe.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.padre.request;

import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * A single or span of PaDRe ID(s) (e.g. <code>pdrAo.001.002.000000006</code> or
 * <code>pdrAo.001.002.000000007-pdrAo.001.002.000000008</code>.
 * @author Fabian Körner
 */
public class PadreID implements Comparable<PadreID>
{

	private static final String REGEX = "([a-z]{5})[.]([0-9a-f]{3})[.]([0-9a-f]{3})[.]([0-9a-f]{9})";
	public static final Pattern PATTERN_SINGLE = Pattern.compile("^" + REGEX + "$", Pattern.CASE_INSENSITIVE);
	public static final Pattern PATTERN_SPAN = Pattern.compile("^[ ]?(" + REGEX + ")[ ]?-[ ]?(" + REGEX + ")?[ ]?$", Pattern.CASE_INSENSITIVE);

	private String _input;

	private boolean _isSpan;
	private PadreID _lowerBound;
	private PadreID _upperBound;

	private String _partType;
	private String _partInstance;
	private String _partStore;
	private String _partObject;
	private int _valueInstance;
	private int _valueStore;
	private long _valueObject;

	/**
	 * Constructor with provided string representation of the ID.
	 * @param input Input string.
	 * @throws PadreIDException Invalid input string.
	 */
	public PadreID(final String input) throws PadreIDException
	{
		_input = input.trim();
		this.parse();
	}

	/**
	 * Returns object type part as <code>java.lang.String</code>.
	 * @return Object type as String.
	 */
	public final String getPartType()
	{
		return _partType;
	}

	/**
	 * Returns instance ID part as <code>java.lang.String</code>.
	 * @return Instance ID as String.
	 */
	public final String getPartInstance()
	{
		return _partInstance;
	}

	/**
	 * Returns store ID part as <code>java.lang.String</code>.
	 * @return Store ID as String.
	 */
	public final String getPartStore()
	{
		return _partStore;
	}

	/**
	 * Returns object ID part as <code>java.lang.String</code>.
	 * @return Object ID as String.
	 */
	public final String getPartObject()
	{
		if (_isSpan)
		{
			throw new IllegalStateException("Can't get object ID from span: " + this.toString());
		}
		else
		{
			return _partObject;
		}
	}

	/**
	 * Returns instance ID.
	 * @return Instance ID.
	 */
	public final int getValueInstance()
	{
		return _valueInstance;
	}

	/**
	 * Returns store ID.
	 * @return Store ID.
	 */
	public final int getValueStore()
	{
		return _valueStore;
	}

	/**
	 * Returns object ID (part).
	 * @return Object ID.
	 */
	public final long getValueObject()
	{
		if (_isSpan)
		{
			throw new IllegalStateException("Can't get object ID from span: " + this.toString());
		}
		else
		{
			return _valueObject;
		}
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public final String toString()
	{
		StringBuilder sb = new StringBuilder();
		if (_isSpan)
		{
			sb.append(_lowerBound.toString());
			sb.append('-');
			sb.append(_upperBound.toString());
		}
		else
		{
			sb.append(_partType);
			sb.append('.');
			sb.append(_partInstance);
			sb.append('.');
			sb.append(_partStore);
			sb.append('.');
			sb.append(_partObject);
		}
		return sb.toString();
	}

	/**
	 * Checks if the ID is a span.
	 * @return ID is a span.
	 */
	public final boolean isSpan()
	{
		return _isSpan;
	}

	/**
	 * Return the lower bound of a span.
	 * @return Lower bound.
	 */
	public final PadreID getLowerBound()
	{
		return isSpan() ? _lowerBound : this;
	}

	/**
	 * Returns the upper bound of a span.
	 * @return Upper bound.
	 */
	public final PadreID getUpperBound()
	{
		return isSpan() ? _upperBound : this;
	}

	/**
	 * Returns a full list of IDs from a span.
	 * @return List of IDs.
	 * @throws PadreIDException Could not create the list of IDs.
	 */
	public final List<PadreID> getFullSpan() throws PadreIDException
	{
		List<PadreID> result = new Vector<PadreID>();
		if (_isSpan)
		{
			result.add(_lowerBound);
			if (!_lowerBound.equals(_upperBound))
			{
				String prefix = _lowerBound.toString().substring(0, 14);
				long currentValueObject = _lowerBound.getValueObject() + 1L;
				long maxValueObject = _upperBound.getValueObject() - 1L;
				while (currentValueObject <= maxValueObject)
				{
					result.add(new PadreID(prefix + this.addLeadingZerosToId(Long.toHexString(currentValueObject), 9)));
					currentValueObject++;
				}
				result.add(_upperBound);
			}
			return result;
		}
		else
		{
			result.add(this);
		}
		return result;
	}

	/**
	 * Returns the pattern of valid single IDs.
	 * @return Pattern of single IDs.
	 */
	public static Pattern getPatternSingle()
	{
		return PATTERN_SINGLE;
	}

	/**
	 * Returns the pattern of valid ID spans.
	 * @return Pattern of ID spans.
	 */
	public static Pattern getPatternSpan()
	{
		return PATTERN_SPAN;
	}

	private void parse() throws PadreIDException
	{
		Matcher matcherSpan = PATTERN_SPAN.matcher(_input);
		Matcher matcherSingle = PATTERN_SINGLE.matcher(_input);
		if (matcherSpan.find())
		{
			String firstId = matcherSpan.group(1);
			String secondId = matcherSpan.group(6);
			_isSpan = true;
			_lowerBound = new PadreID(firstId);
			_upperBound = new PadreID(secondId);
			_partType = _lowerBound.getPartType();
			_partInstance = _lowerBound.getPartInstance();
			_partStore = _lowerBound.getPartStore();
			_valueInstance = _lowerBound.getValueInstance();
			_valueStore = _lowerBound.getValueStore();
		}
		else if (matcherSingle.find())
		{
			_isSpan = false;
			_partType = matcherSingle.group(1);
			_partInstance = matcherSingle.group(2);
			_partStore = matcherSingle.group(3);
			_partObject = matcherSingle.group(4);
			_valueInstance = Integer.parseInt(_partInstance, 16);
			_valueStore = Integer.parseInt(_partStore, 16);
			_valueObject = Long.parseLong(_partObject, 16);
		}
		else
		{
			throw new PadreIDException("invalid PaDRe ID: " + _input);
		}
		this.validate();
	}

	private void validate() throws PadreIDException
	{
		if (_isSpan)
		{
			this.validateSpan();
		}
		else
		{
			this.validateSingleId();
		}
	}

	private void validateSpan() throws PadreIDException
	{
		if (!_lowerBound.getPartType().equals(_upperBound.getPartType()))
		{
			throw new PadreIDException("type of lower and upper bound must be the same");
		}
		if (_lowerBound.getValueInstance() != _upperBound.getValueInstance())
		{
			throw new PadreIDException("instance of lower and upper bound must be the same");
		}
		if (_lowerBound.getValueStore() != _upperBound.getValueStore())
		{
			throw new PadreIDException("store of lower and upper bound must be the same");
		}
		if (_lowerBound.getValueObject() > _upperBound.getValueObject())
		{
			throw new PadreIDException("object of lower bound is greater than upper bound");
		}
	}

	private void validateSingleId() throws PadreIDException
	{
		if (_valueInstance == 0)
		{
			throw new PadreIDException("instance must not be zero");
		}
		if (_valueStore == 0)
		{
			throw new PadreIDException("store must not be zero");
		}
		if (_valueObject == 0)
		{
			throw new PadreIDException("object must not be zero");
		}
	}

	private String addLeadingZerosToId(final String id, final int targetLength)
	{
		if (id.length() > targetLength)
		{
			throw new IllegalArgumentException("ID already longer than target length: " + id + ", " + Integer.toString(targetLength));
		}
		StringBuilder sb = new StringBuilder(id);
		while (sb.length() < targetLength)
		{
			sb.insert(0, '0');
		}
		return sb.toString();
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	@Override
	public final boolean equals(final Object arg0)
	{
		return this.hashCode() == arg0.hashCode();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public final int hashCode()
	{
		int result = 1;

		if (_isSpan)
		{
			result = result * 11 + _lowerBound.hashCode();
			result = result * 13 + _upperBound.hashCode();
		}
		else
		{
			result = result * 11 + _partType.hashCode();
			result = result * 13 + _valueInstance;
			result = result * 17 + _valueStore;
			result = result * 19 + (int) (_valueObject & 0xFFFFFFFF);
		}

		return result;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	@Override
	public final int compareTo(final PadreID arg0)
	{
		if (_isSpan || arg0.isSpan())
		{
			throw new IllegalArgumentException("spans of PaDRe IDs are not compareable");
		}
		else
		{
			boolean typeEquals = _partType.equals(arg0.getPartType());
			boolean instanceEquals = (_valueInstance == arg0.getValueInstance());
			boolean projectEquals = (_valueStore == arg0.getValueStore());

			if (typeEquals && instanceEquals && projectEquals)
			{
				return PadreID.safeLongToInt(_valueObject - arg0.getValueObject());
			}
			else
			{
				throw new IllegalArgumentException("PaDRe IDs of different types, instances, or stores are not compareable");
			}
		}
	}

	private static int safeLongToInt(final long value)
	{
		if (value < Integer.MIN_VALUE || value > Integer.MAX_VALUE)
		{
			throw new IllegalArgumentException(value + "cannot safely cast " + Long.toString(value) + " to int");
		}
		return (int) value;
	}

	/**
	 * Creates a PaDRe ID without throwing Exceptions. Validity of the provided
	 * input string must be ensured before calling this method. In case, of an
	 * <code>null</code> will be returned.
	 * @param id ID string.
	 * @return PadreID or <code>null</code>.
	 */
	public static PadreID create(final String id)
	{
		PadreID result;
		try
		{
			result = new PadreID(id);
		}
		catch (PadreIDException e)
		{
			return null;
		}
		return result;
	}
}
