/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.utils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Common way of signaling problems with any kind of interactions with the repository.
 * @author Fabian Körner
 */
public class RepositoryException extends Exception
{

	private static final long serialVersionUID = -6555979550628371877L;

	private static final Pattern PATTERN_COMBINED_MESSAGE = Pattern.compile("^\\[([A-Z]+?)\\]-(.*?)$");

	private String _message;

	/**
	 * @see java.lang.Exception#Exception(String)
	 * @param message Error message.
	 */
	public RepositoryException(final String message)
	{
		this(ErrorSource.UNKNOWN, message);
	}

	/**
	 * Includes label of trouble-causing PDR module.
	 * @param errorSource PDR source of error.
	 * @param message Error message.
	 */
	public RepositoryException(final ErrorSource errorSource, final String message)
	{
		super(message);
		_message = RepositoryException.composeMessage(errorSource, message);
	}

	/**
	 * @see java.lang.Exception#Exception(Throwable)
	 * @param cause The cause.
	 */
	public RepositoryException(final Throwable cause)
	{
		this(ErrorSource.UNKNOWN, cause);
	}

	/**
	 * Includes label of trouble-causing PDR module.
	 * @param errorSource PDR source of error.
	 * @param cause The cause.
	 */
	public RepositoryException(final ErrorSource errorSource, final Throwable cause)
	{
		super(cause);
		_message = RepositoryException.composeMessage(errorSource, cause.getMessage());
	}

	/**
	 * @see java.lang.Exception#Exception(String, Throwable)
	 * @param message Error message.
	 * @param cause The cause.
	 */
	public RepositoryException(final String message, final Throwable cause)
	{
		this(ErrorSource.UNKNOWN, message, cause);
	}

	/**
	 * Includes label of trouble-causing PDR module.
	 * @param errorSource PDR source of error.
	 * @param message Error message.
	 * @param cause The cause.
	 */
	public RepositoryException(final ErrorSource errorSource, final String message, final Throwable cause)
	{
		super(message, cause);
		_message = RepositoryException.composeMessage(errorSource, message);
	}

	private static String composeMessage(final ErrorSource errorSource, final String message)
	{
		StringBuilder sb = new StringBuilder((null == message) ? "" : message);
		Matcher matcher = PATTERN_COMBINED_MESSAGE.matcher(sb.toString());
		if (!matcher.find())
		{
			sb.insert(0, "]-");
			sb.insert(0, errorSource.toString());
			sb.insert(0, "[");
		}
		return sb.toString();
	}

	@Override
	public final String getMessage()
	{
		return _message;
	}

	/**
	 * Extracts error source from error message.
	 * @return Error source.
	 */
	public final ErrorSource getErrorSourceFromMessage()
	{
		Matcher matcher = PATTERN_COMBINED_MESSAGE.matcher(_message);
		if (matcher.find())
		{
			String errorSource = matcher.group(1);
			if (errorSource.equals(ErrorSource.VALIDATION.toString()))
			{
				return ErrorSource.VALIDATION;
			}
			else if (errorSource.equals(ErrorSource.STORAGE.toString()))
			{
				return ErrorSource.STORAGE;
			}
			else if (errorSource.equals(ErrorSource.INDEX.toString()))
			{
				return ErrorSource.INDEX;
			}
			else
			{
				return ErrorSource.UNKNOWN;
			}
		}
		else
		{
			return ErrorSource.UNKNOWN;
		}
	}

	/**
	 * Distinct error PDR sources.
	 * @author Fabian Körner
	 */
	public static enum ErrorSource
	{
		VALIDATION
		{
			@Override
			public String toString()
			{
				return "VALIDATION";
			}
		},
		STORAGE
		{
			@Override
			public String toString()
			{
				return "STORAGE";
			}
		},
		INDEX
		{
			@Override
			public String toString()
			{
				return "INDEX";
			}
		},
		UNKNOWN
		{
			@Override
			public String toString()
			{
				return "UNKNOWN";
			}
		},
	}
}
