/*
 *	Copyright 2012-2014 Berlin-Brandenburgische Akademie der Wissenschaften
 *
 *	This file is part of PDR-LIB.
 *
 *	PDR-LIB is free software: you can redistribute it and/or modify it under
 *	the terms of the GNU Lesser General Public License as published by the
 *	Free Software Foundation, either version 3 of the License, or (at your
 *	option) any later version.
 *
 *	PDR-LIB is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *	FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 *	more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with PDR-LIB.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bbaw.pdr.utils;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.XMLConstants;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Validation of PDR objects before ingestion or modification.
 * @author Fabian Körner
 */
public final class Validator
{

	private static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	private static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";

	private static final Pattern PATTERN_RECORD_ELEMENT = Pattern.compile("<record>(.+?)</record>");
	private static final Pattern PATTERN_REVISION_ELEMENTS = Pattern.compile("<revision.+?timestamp=\"(.+?)\".*?/>");
	private static final Pattern PATTERN_RECORD_INFO_ELEMENT = Pattern.compile("<recordInfo>(.+?)</recordInfo>");
	private static final Pattern PATTERN_CREATION_OR_CHANGE_DATE_ELEMENTS = Pattern.compile("<record(Creation|Change)Date\\s.+?>(.+?)</record(\\1)Date>");
	private static final Pattern PATTERN_TIME_ELEMENT = Pattern.compile("<time\\s.+?>(.+?)</time>");
	private static final Pattern PATTERN_DATE_ELEMENT = Pattern.compile("<date\\s.*?(when|from|to|notBefore|notAfter)=[\"](.+?)[\"].*? ></date>");

	private static final SimpleDateFormat DATE_FORMAT_REVISION_TIMESTAMPS = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

	private static final SimpleDateFormat[] VALID_DATETIME_FORMATS = {
		new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss"),
		new SimpleDateFormat("yyyy-MM-dd'T'HH:mm"),
		new SimpleDateFormat("yyyy-MM-dd"),
		new SimpleDateFormat("yyyy-MM"),
		new SimpleDateFormat("yyyy"),
	};

	/**
	 * Hidden default constructor.
	 */
	private Validator()
	{
	}

	/**
	 * Validates the provided PDR object against XMLSchema and expected content
	 * and its format.
	 * @param object PDR object to be validated.
	 * @throws ValidationException Validation failed.
	 */
	public static void validate(final PDRObject object) throws ValidationException
	{
		try
		{
			Validator.validateXMLSchema(object);
		}
		catch (Exception cause)
		{
			throw new ValidationException(object.getModifiedIdentifier().toString() + ": xml schema validation failed: " + cause.getMessage(), cause);
		}

		try
		{
			Validator.validateTimeFormats(object);
		}
		catch (Exception cause)
		{
			throw new ValidationException(object.getModifiedIdentifier().toString() + ": time formats validation failed: " + cause.getMessage(), cause);
		}
	}

	private static void validateXMLSchema(final PDRObject object) throws SAXException, ParserConfigurationException, IOException
	{
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		factory.setNamespaceAware(true);

		SAXParser parser = factory.newSAXParser();
		parser.setProperty(JAXP_SCHEMA_LANGUAGE, XMLConstants.W3C_XML_SCHEMA_NS_URI);

		XMLReader reader = parser.getXMLReader();
		reader.setErrorHandler(new XMLSchemaErrorHandler());

		String xmlContent = object.getXMLContent();

		if (PDRType.REFERENCE == object.getType())
		{
			parser.setProperty(JAXP_SCHEMA_SOURCE, "http://pdr.bbaw.de/schema/rodl_mods.xsd");
		}

		reader.parse(new InputSource(new ByteArrayInputStream(xmlContent.getBytes("UTF-8"))));
	}

	private static void validateTimeFormats(final PDRObject object) throws ParseException, ValidationException
	{
		Validator.validateRecordRevisionTimestamp(object);
		if (PDRType.ASPECT == object.getType())
		{
			Validator.validateTimeDimDateTimes(object);
			Validator.validateNotificationDateTimes(object);
		}
	}

	private static void validateRecordRevisionTimestamp(final PDRObject object) throws ParseException
	{
		if (PDRType.REFERENCE == object.getType())
		{
			Matcher matcherRecord = PATTERN_RECORD_INFO_ELEMENT.matcher(object.getXMLContent());
			matcherRecord.find();
			Matcher matcherRevisions = PATTERN_CREATION_OR_CHANGE_DATE_ELEMENTS.matcher(matcherRecord.group(1));
			while (matcherRevisions.find())
			{
				DATE_FORMAT_REVISION_TIMESTAMPS.parse(matcherRevisions.group(2));
			}
		}
		else
		{
			Matcher matcherRecord = PATTERN_RECORD_ELEMENT.matcher(object.getXMLContent());
			matcherRecord.find();
			Matcher matcherRevisions = PATTERN_REVISION_ELEMENTS.matcher(matcherRecord.group(1));
			while (matcherRevisions.find())
			{
				DATE_FORMAT_REVISION_TIMESTAMPS.parse(matcherRevisions.group(1));
			}
		}
	}

	private static void validateTimeDimDateTimes(final PDRObject object) throws ValidationException
	{
		Matcher matcher = PATTERN_TIME_ELEMENT.matcher(object.getXMLContent());
		while (matcher.find())
		{
			Validator.validateDateTimeFormat(matcher.group(1));
		}
	}

	private static void validateNotificationDateTimes(final PDRObject object) throws ValidationException
	{
		Matcher matcher = PATTERN_DATE_ELEMENT.matcher(object.getXMLContent());
		while (matcher.find())
		{
			Validator.validateDateTimeFormat(matcher.group(2));
		}
	}

	private static void validateDateTimeFormat(final String dateTime) throws ValidationException
	{
		try
		{
			Date date = null;
			int index = 0;
			while (null == date)
			{
				try
				{
					date = VALID_DATETIME_FORMATS[index++].parse(dateTime);
				}
				catch (ParseException pe)
				{
					continue;
				}
			}
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			throw new ValidationException("invalid datetime format: " + dateTime);
		}
	}
}
